Package: methimpute
Type: Package
Title: Imputation-guided re-construction of complete methylomes from
        WGBS data
Version: 1.34.0
Author: Aaron Taudt
Maintainer: Aaron Taudt <aaron.taudt@gmail.com>
Description: This package implements functions for calling methylation for all cytosines in the genome.
Depends: R (>= 3.5.0), GenomicRanges, ggplot2
Imports: Rcpp (>= 0.12.4.5), methods, utils, grDevices, stats,
        GenomeInfoDb, IRanges, Biostrings, reshape2, minpack.lm,
        data.table
Suggests: knitr, BiocStyle
LinkingTo: Rcpp
License: Artistic-2.0
LazyLoad: yes
VignetteBuilder: knitr
RoxygenNote: 6.1.1
biocViews: ImmunoOncology, Software, DNAMethylation, Epigenetics,
        HiddenMarkovModel, Sequencing, Coverage
git_url: https://git.bioconductor.org/packages/methimpute
git_branch: RELEASE_3_23
git_last_commit: 5c8ef59
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: yes
Packaged: 2026-04-28 23:10:59 UTC; biocbuild
