# Generated by roxygen2: do not edit by hand

S3method(print,methimputeBinomialHMM)
export(binCounts)
export(binMethylome)
export(binPositions)
export(binomialTestMethylation)
export(callMethylation)
export(callMethylationSeparate)
export(collapseBins)
export(distanceCorrelation)
export(estimateTransDist)
export(exportMethylome)
export(extractCytosinesFromFASTA)
export(getDistinctColors)
export(getStateColors)
export(importBSMAP)
export(importBSSeeker)
export(importBismark)
export(importMethylpy)
export(inflateMethylome)
export(loadFromFiles)
export(plotConvergence)
export(plotEnrichment)
export(plotHistogram)
export(plotPosteriorDistance)
export(plotScatter)
export(plotTransitionProbs)
import(GenomeInfoDb)
import(GenomicRanges)
import(IRanges)
import(Rcpp)
import(ggplot2)
importFrom(Biostrings,readDNAStringSet)
importFrom(Biostrings,reverseComplement)
importFrom(Biostrings,vcountPattern)
importFrom(Biostrings,vmatchPattern)
importFrom(data.table,fread)
importFrom(data.table,fwrite)
importFrom(grDevices,col2rgb)
importFrom(grDevices,rgb2hsv)
importFrom(methods,as)
importFrom(methods,is)
importFrom(minpack.lm,nlsLM)
importFrom(reshape2,melt)
importFrom(stats,aggregate)
importFrom(stats,coefficients)
importFrom(stats,dbinom)
importFrom(stats,dist)
importFrom(stats,na.omit)
importFrom(stats,p.adjust)
importFrom(stats,pbinom)
importFrom(utils,read.table)
useDynLib(methimpute)
