

#' methIMPUTE: Imputation-guided methylation status calling for WGBS-seq
#'
#' \pkg{methimpute} is an R-package for methylation status calling in Whole-Genome Bisulfite-sequencing (WGBS-seq) data. Its powerful Hidden Markov model implementation enables imputation of methylation status calls for cytosines without any coverage.
#'
#' Please read the vignette for a tutorial on how to use this package. You can do this by typing \code{browseVignettes("methimpute")}. Here is an overview of all \code{\link{plotting}} functions.
#'
#' @author Aaron Taudt
#' @docType package
#' @name methimpute-package
#' @aliases methimpute
NULL
