# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

fitBinomialTestHMMcontextTransition <- function(counts_total, counts_meth, context, transitionContext, distances, params, algorithm, update_procedure) {
    .Call('_methimpute_fitBinomialTestHMMcontextTransition', PACKAGE = 'methimpute', counts_total, counts_meth, context, transitionContext, distances, params, algorithm, update_procedure)
}

cleanup <- function() {
    invisible(.Call('_methimpute_cleanup', PACKAGE = 'methimpute'))
}

fitHMM <- function(counts, distances, params, algorithm) {
    .Call('_methimpute_fitHMM', PACKAGE = 'methimpute', counts, distances, params, algorithm)
}

