#' @useDynLib methimpute
#' @import Rcpp
#' @import IRanges
#' @import GenomeInfoDb
#' @import GenomicRanges
#' @importFrom methods as is
NULL

# =======================================================
# Some global variables that can be used in all functions
# =======================================================
